--[[ include used code ]]
require "stgTools"
require "settings"

--[[ local var - dont change things plx ]]
local PlayerLife_okMin = 80
local PlayerLife_dangerMin = 40
local brightness = 2

--[[ playersFrame actions ]]
local function create_playersFlow(player, playerindex)
	if glob.cfg.privateTableView then
		local myTable = player.gui.left.tfrPlayers.add( { type="table", colspan= 2, name="playersFlow", style="tms_playerTable" } )

		myTable.add( { type= "label", style= "tms_playerLabel_caption", caption= "name" } )
		myTable.add( { type= "label", style= "tms_playerLabel_caption", caption= "status" } )

		for locPlayerIndex, locPlayer in pairs(game.players) do
			--$$ index and name 
			local myElement = myTable.add( { type= "label", style= "tms_playerLabel",
					caption= string.format( "%d) %s", locPlayerIndex, locPlayer.name ) } )
			myElement.style.font = "default-bold"
			local colorTable = {}
			colorTable.r = locPlayer.color["r"]
			colorTable.g = locPlayer.color["g"]
			colorTable.b = locPlayer.color["b"]
			--colorTable.a = locPlayer.color.a
			myElement.style.fontcolor = colorTable--$$ I wants copyTable
			--myElement.style.fontcolor.a = 1

			--$$ controller-status
			if locPlayer.character then
				myElement = myTable.add( { type= "label", style= "tms_playerLabel", caption= "alive" } )
			else
				myElement = myTable.add( { type= "label", style= "tms_playerLabel", caption= "dead" } )
				myElement.style.fontcolor = {r=1} --$$ red
			end
		end
	else
		local myTable = player.gui.left.tfrPlayers.add( { type="table", colspan= 3, name="playersFlow", style="tms_playerTable" } )

		myTable.add( { type= "label", style= "tms_playerLabel_caption", caption= "name" } )
		myTable.add( { type= "label", style= "tms_playerLabel_caption", caption= "life" } )
		myTable.add( { type= "label", style= "tms_playerLabel_caption", caption= "vehicle" } )

		for locPlayerIndex, locPlayer in pairs(game.players) do
			--$$ index and name 
			local myElement = myTable.add( { type= "label", style= "tms_playerLabel",
					caption= string.format( "%d) %s", locPlayerIndex, locPlayer.name ) } )
			myElement.style.font = "default-bold"
			local colorTable = {}
			colorTable.r = locPlayer.color["r"]
			colorTable.g = locPlayer.color["g"]
			colorTable.b = locPlayer.color["b"]
			--colorTable.a = locPlayer.color.a
			myElement.style.fontcolor = colorTable--$$ I wants copyTable

			--$$ character-life 
			if locPlayer.character then
				myElement = myTable.add( { type= "label", style= "tms_playerLabel",
						caption= string.format( "%d", locPlayer.character.health ) } )
				if locPlayer.character.health < PlayerLife_dangerMin then -- if semi dead
					myElement.style.fontcolor = {r=1,g=0,b=0} --$$ red
				elseif locPlayer.character.health < PlayerLife_okMin  then -- if in danger but ok
					myElement.style.fontcolor = {r=1,g=0.7,b=0} --$$ orange
				else -- ok
					myElement.style.fontcolor = {r=0.2,g=1,b=0.2} --$$ green
				end

			else
				myElement = myTable.add( { type= "label", style= "tms_playerLabel", caption= "hp:N/A" } )
			end

			--$$ vehicle
			myElement = myTable.add( { type= "label", style= "tms_playerLabel" } )
			if not locPlayer.vehicle then
				myElement.caption = "walking"
			else
				myElement.caption = string.format( "%s: %dhp", locPlayer.vehicle.name, locPlayer.vehicle.health )
			end
		end
	end
end

local function create_playersFrame(player, playerindex)
	local myFrame = player.gui.left.add({ type="frame", name= "tfrPlayers", 
			style="tms_frame_compact", caption="Players: "..#game.players, direction="horizontal" })
	--		style="tms_flow_compact", direction="horizontal" })
	myFrame.style.bottompadding = 4	
	myFrame.style.rightpadding = 2	

	create_playersFlow(player, playerindex)
	glob.playersFrame_open[playerindex] = true
end

local function playersButton_changeCaption(player, caption)
	stg_setActionElementCaption(player, "mptbnPlayers", caption)
end

local function open_or_close_playersFrame(player, playerindex)
	if player.gui.left.tfrPlayers then 
		--$$close
		player.gui.left.tfrPlayers.destroy()
		playersButton_changeCaption(player, "Players +")
		glob.playersFrame_open[playerindex] = false
	else
		--$$open
		create_playersFrame(player, playerindex)
		playersButton_changeCaption(player, "Players -")
	end
end

local function playersFrame_openRenew(player, playerindex)
	if player.gui.left.tfrPlayers then
		player.gui.left.tfrPlayers.destroy()
	end
	open_or_close_playersFrame(player, playerindex)
end

--[[ PUBLIC stuff ]]
function tf_close_playersFrame(player, playerindex)
	if player.gui.left.tfrPlayers then
		player.gui.left.tfrPlayers.destroy()
	end

	playersButton_changeCaption(player, "Players +")
	glob.playersFrame_open[playerindex] = false
end

function tf_playersButton_onRemove(playerindex)
	local player = game.players[playerindex]
	stg_destroyActionElement(player, "mptbnPlayers")
	local tfrPlayers = player.gui.left.tfrPlayers
	if tfrPlayers then
		tfrPlayers.destroy()
	end
end

function tf_playersButton_onCreate(playerindex)
	--if not glob.cfg.showPlayersButtton then return end
	--$$ playerSettings
	--sf_check_playerCfg(playerindex)

	--$$ button
	local player = game.players[playerindex]
	if glob.playerCfg[playerindex].showPlayersButtton then
		local stGui = player.gui.top.superTopGui
		if stGui and stGui.stg_actionPanel and not stGui.stg_actionPanel.mptbnPlayers then
			stGui.stg_actionPanel.add( { type= "button", name= "mptbnPlayers", style= "tms_buttonAlpha", caption= "Players +" } )
		end
	end
	--$$ frame
	if not glob.playersFrame_open then 
		glob.playersFrame_open = {}
		glob.playersFrame_open[playerindex] = false
	elseif glob.playersFrame_open[playerindex] then
		playersFrame_openRenew(player, playerindex)
	end
end

function tf_playersButton_onUpdate(playerindex)
 	--if not glob.cfg.showPlayersButtton then return end
	local player = game.players[playerindex]
	local tfrPlayers = player.gui.left.tfrPlayers
	if tfrPlayers and glob.playersFrame_open[playerindex] then
		if tfrPlayers.playersFlow then
			tfrPlayers.playersFlow.destroy()
		end
		--player.gui.left.tfrPlayers.caption = string.format("Players: %d", #game.players )
		create_playersFlow(player, playerindex)
	end
end

local function pushPlayerTableUpdate(player, playerindex, updatePlayerOnly)
	if updatePlayerOnly then
		tf_playersButton_onUpdate(playerindex)
	else
		for locPlayerIndex, locPlayer in pairs(game.players) do
			tf_playersButton_onUpdate(locPlayerIndex)
		end
	end
end

--[ event onguiclick ]
game.onevent(defines.events.onguiclick, 
function(event)
	local player = game.players[event.playerindex]
	--$$ open/close playerTable
	if event.element.name == "mptbnPlayers" then
		open_or_close_playersFrame(player, event.playerindex)

	--$$ open setting 
	elseif event.element.name == MOD_SETTINGS_BUTTON then
		tf_pB_settings_openUpdate(player, event.playerindex)

	--$$ Select Settings table
	elseif event.element.name == "mtbnSetsPlyr" then
		local cfg = glob.playerCfg[event.playerindex]
		if cfg and not cfg.viewPlayerSettings then
			cfg.viewPlayerSettings = true
		end 
		tf_pB_settings_openUpdate(player, event.playerindex)

	elseif event.element.name == "mtbnSetsGlob" then
		local cfg = glob.playerCfg[event.playerindex]
		if cfg and cfg.viewPlayerSettings then
			cfg.viewPlayerSettings = false
		end 
		tf_pB_settings_openUpdate(player, event.playerindex)

	--$$ Toggle Settings
	elseif event.element.name == "mptbnTglShowBtn" then --$$ Toggle Show Players+ Button
		local cfg = glob.playerCfg[event.playerindex]
		local updatePlayerOnly = true
		if not cfg or not cfg.viewPlayerSettings then
			cfg = glob.cfg
			updatePlayerOnly = false
		end 

		cfg.showPlayersButtton = not cfg.showPlayersButtton
		tf_pB_settings_openUpdate(player, event.playerindex)
		if updatePlayerOnly then
			if not cfg.showPlayersButtton then
				stg_destroyActionElement(player, "mptbnPlayers")
			else
				stg_update(player)
			end
		else 
			--$$ set settings for all players and update gui
			if not cfg.showPlayersButtton then
				for locPlayerIndex, locPlayer in pairs(game.players) do
					glob.playerCfg[locPlayerIndex].showPlayersButtton = cfg.showPlayersButtton
					stg_destroyActionElement(locPlayer, "mptbnPlayers")
				end
			else
				for locPlayerIndex, locPlayer in pairs(game.players) do
					glob.playerCfg[locPlayerIndex].showPlayersButtton = cfg.showPlayersButtton
					stg_update(locPlayer)
				end
			end
		end

	elseif event.element.name == "mptbnTglPvtMode" then --$$ Toggle privateTableView
		local cfg = glob.playerCfg[event.playerindex]
		local updatePlayerOnly = true
		if not cfg or not cfg.viewPlayerSettings then
			cfg = glob.cfg
			updatePlayerOnly = false
		end 

		cfg.privateTableView = not cfg.privateTableView
		pushPlayerTableUpdate(player, event.playerindex, updatePlayerOnly)
		tf_pB_settings_openUpdate(player, event.playerindex)

	elseif event.element.name == "mptbnTglPrintMode" then --$$ Toggle printMode
		local cfg = glob.playerCfg[event.playerindex]
		local updatePlayerOnly = true
		if not cfg or not cfg.viewPlayerSettings then
			cfg = glob.cfg
			updatePlayerOnly = false
		end 

		cfg.printMode = not cfg.printMode
		tf_pB_settings_openUpdate(player, event.playerindex)
	end
end)

